/*****************************************************

Project : ks0108Test
Version : inportierte ks0108 Routinen von Fabian Maximilian Thiele
Date    : 11.02.2006

Chip type           : ATmega128
Program type        : Application
Clock frequency     : 14,745600 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 1024
*****************************************************/

#include <mega128.h>  
#include <stdio.h>
#include <delay.h>

//#include "ks0108.h"
#include "Dial_10.h"


// Declare your global variables here

/*
** GLCD Anschluss
**
** LCD 		Pin     Port
**
** D0 		7	   PORTA D0
** D1		8	   PORTA D1
** D2		9	   PORTA D2
** D3		10	   PORTA D3
** D4		11	   PORTA D4
** D5		12	   PORTA D5
** D6		13	   PORTA D6
** D7		14     PORTA D7
**
** RS/D_I	4	   PORTG Bit4     
** R/W		5	   PORTG Bit3
** E		6	   PORTG Bit2
** CS1		15	   PORTG Bit1
** CS2		16	   PORTG Bit0
** Reset	17	   VCC	
*/

#define LCD_DATA_OUT		PORTA	// address of data register 
#define LCD_DATA_IN			PINA	// Data Input Register
#define LCD_DATA_DIR		DDRA	// Data Direction Register for Data Port

#define LCD_CMD_PORT		PORTG   // address of control register 
#define LCD_CMD_DIR			DDRG	// Data Direction Register for Command Port

// Command Port Bits
#define D_I					0x04		// D/I Bit Number
#define R_W					0x03		// R/W Bit Number
#define EN					0x02		// EN Bit Number
#define CSEL1				0x01		// CS1 Bit Number
#define CSEL2				0x00		// CS2 Bit Number  

// Chips
#define CHIP1				1
#define CHIP2				2

// Commands
#define LCD_ON				0x3F
#define LCD_OFF				0x3E
#define LCD_SET_ADD			0x40     	// Y Adresse von 0...63
#define LCD_SET_PAGE		0xB8		// X Adresse von 0...7
#define LCD_DISP_START		0xC0

#define RIGHT 0
#define LEFT 1

// Colors 
#define BLACK		0xFF
#define WHITE		0x00

// Font Indices 
#define FONT_LENGTH			0			// 0
#define FONT_FIXED_WIDTH	2 			// 2
#define FONT_HEIGHT			3			// 3
#define FONT_FIRST_CHAR		4			// 4
#define FONT_CHAR_COUNT		5 			// 5
#define FONT_WIDTH_TABLE	6 			// 6 

typedef struct
{
unsigned char x;
unsigned char y;
unsigned char page;
} lcdCoord;

typedef unsigned char (*ks0108FontCallback)(flash unsigned char*);


// Makros
#define ks0108ClearScreen() {ks0108FillRect(0, 0, 127, 63, WHITE);}    
#define ks0108DrawVertLine(x, y, length, color) {ks0108FillRect(x, y, 0, length, color);}
#define ks0108DrawHoriLine(x, y, length, color) {ks0108FillRect(x, y, length, 0, color);}
#define ks0108DrawCircle(xCenter, yCenter, radius, color) {ks0108DrawRoundRect(xCenter-radius, yCenter-radius, 2*radius, 2*radius, radius, color);}



extern void ks0108Enable(void);      
extern void ks0108WriteCommand(unsigned char cmd, unsigned char chip);
extern void ks0108WriteData (unsigned char data);  
extern void ks0108GotoXY(unsigned char x, unsigned char y);
inline extern unsigned char ks0108ReadData(void);
extern unsigned char ks0108DoReadData(unsigned char first); 
extern void ks0108Init(unsigned char invert);
  

// Graphic Functions
extern void ks0108DrawLine(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char color);
extern void ks0108FillRect(unsigned char x, unsigned char y, unsigned char width, unsigned char height, unsigned char color);

// Font Functions
extern unsigned char ks0108ReadFontData(flash unsigned char* ptr);
extern void ks0108SelectFont(flash unsigned char* font, ks0108FontCallback callback, unsigned char color);
extern int ks0108PutChar(char c);
extern void ks0108Puts(char* str);


lcdCoord		ks0108Coord;
unsigned char	ks0108Inverted=0;

ks0108FontCallback	    ks0108FontRead;
unsigned char		    ks0108FontColor;
flash unsigned char*	ks0108Font;

// GLCD Basis-Funktionen

void ks0108Enable(void)
{
LCD_CMD_PORT |= 0x01 << EN;						     
delay_us(1);                                    // EN high level mit: min. 450ns
LCD_CMD_PORT &= ~(0x01 << EN);
}

void ks0108WriteCommand(unsigned char cmd, unsigned char chip)
{
if(chip == CHIP1)
	{ 
	LCD_CMD_PORT &= ~(0x01 << CSEL2);			// deselect chip 2
	LCD_CMD_PORT |= 0x01 << CSEL1;				// select chip 1
	}
else
	if(chip == CHIP2)
		{
		LCD_CMD_PORT &= ~(0x01 << CSEL1);		// deselect chip 1
		LCD_CMD_PORT |= 0x01 << CSEL2;			// select chip 2
		}
LCD_CMD_PORT &= ~(0x01 << D_I);					// D/I = 0   
delay_us(1);
LCD_CMD_PORT &= ~(0x01 << R_W);					// R/W = 0
LCD_DATA_DIR = 0xFF;				   			// data port is output
LCD_DATA_OUT = cmd;					   			// write command
ks0108Enable();
LCD_DATA_OUT = 0x00;                     	
} 

void ks0108WriteData (unsigned char data)
{
unsigned char displayData, yOffset, cmdPort;
#ifdef DEBUG
volatile unsigned long int i;
for(i=0; i<5000; i++);
#endif
if(ks0108Coord.x >= 128)
	return;
if(ks0108Coord.x < 64)
	{
	LCD_CMD_PORT &= ~(0x01 << CSEL2);			// deselect chip 2
	LCD_CMD_PORT |= 0x01 << CSEL1;				// select chip 1
	}
else
	if( ks0108Coord.x >= 64)
		{
		LCD_CMD_PORT &= ~(0x01 << CSEL1);		// deselect chip 1
		LCD_CMD_PORT |= 0x01 << CSEL2;			// select chip 2
		}
	if( ks0108Coord.x == 64)		   			// chip2 X-address = 0 
		ks0108WriteCommand(LCD_SET_ADD, CHIP2);
	LCD_CMD_PORT |= 0x01 << D_I;				// D/I = 1   
	delay_us(1);
	LCD_CMD_PORT &= ~(0x01 << R_W);				// R/W = 0
	LCD_DATA_DIR = 0xFF;						// data port is output
	yOffset = ks0108Coord.y%8; 
		if(yOffset != 0)
		{
		// first page 
    	cmdPort = LCD_CMD_PORT;					// save command port
		displayData = ks0108ReadData();
		
		LCD_CMD_PORT = cmdPort;					// restore command port
		LCD_DATA_DIR = 0xFF;					// data port is output
		
		displayData |= data << yOffset;
		if(ks0108Inverted)
			displayData = ~displayData;
		LCD_DATA_OUT = displayData;				// write data
		ks0108Enable();							// enable
		
		// second page
		ks0108GotoXY(ks0108Coord.x, ks0108Coord.y+8);
		
		displayData = ks0108ReadData();
		
		LCD_CMD_PORT = cmdPort;				   	// restore command port
		LCD_DATA_DIR = 0xFF;				  	// data port is output
		
		displayData |= data >> (8-yOffset);
		if(ks0108Inverted)
			displayData = ~displayData;
		LCD_DATA_OUT = displayData;			  	// write data
		ks0108Enable();						  	// enable
		ks0108GotoXY(ks0108Coord.x+1, ks0108Coord.y-8);
		}
	 	else
	 		{
			if(ks0108Inverted)
			    data = ~data;
			LCD_DATA_OUT = data;				// write data
			ks0108Enable();						// enable
			ks0108Coord.x++;
			}
		LCD_DATA_OUT = 0x00;
}

void ks0108GotoXY(unsigned char x, unsigned char y)
{
unsigned char chip = CHIP1;
unsigned char cmd;
if(x > 127)
	 x = 0;				   				        // prfen ob Koordinaten richtig sind
if(y > 63)
	y = 0;
ks0108Coord.x = x;						        // save new coordinates 
ks0108Coord.y = y;
ks0108Coord.page = y/8;
if(x >= 64)
	{					   				        // select the right chip 
	x -= 64;
	chip = CHIP2;
	}
cmd = LCD_SET_ADD | x;
ks0108WriteCommand(cmd, chip);			        // set x address on active chip 
cmd = LCD_SET_PAGE | ks0108Coord.page;	        // set y address on both chips 
ks0108WriteCommand(cmd, CHIP1);
ks0108WriteCommand(cmd, CHIP2);
}

extern unsigned char ks0108ReadData(void)
{
ks0108DoReadData(1);				            // dummy read 
return ks0108DoReadData(0);			            // "real" read 
}

unsigned char ks0108DoReadData(unsigned char first)
{
unsigned char data;
volatile unsigned char i; 		                //mit volatile achtung variable durch interrupt vernderbar 

LCD_DATA_OUT = 0x00;
LCD_DATA_DIR = 0x00;			                // data port is input 

if(ks0108Coord.x < 64)
	{
	LCD_CMD_PORT &= ~(0x01 << CSEL2);			// deselect chip 2
	LCD_CMD_PORT |= 0x01 << CSEL1;				// select chip 1
	}
else
	if(ks0108Coord.x >= 64)
		{
		LCD_CMD_PORT &= ~(0x01 << CSEL1);		// deselect chip 1
		LCD_CMD_PORT |= 0x01 << CSEL2;			// select chip 2
		}
	if(ks0108Coord.x == 64 && first)            // chip2 X-address = 0
		{
		ks0108WriteCommand(LCD_SET_ADD, CHIP2); // wuff wuff
		}
	
	LCD_CMD_PORT |= 0x01 << D_I;				// D/I = 1
	LCD_CMD_PORT |= 0x01 << R_W;				// R/W = 1
	
	LCD_CMD_PORT |= 0x01 << EN;
	delay_us(1);	
 	data = LCD_DATA_IN;							// read Data			 
	
	LCD_CMD_PORT &= ~(0x01 << EN);
 	for(i=0; i<8; i++);							// a little delay loop (faster than reading the busy flag)
	
	LCD_DATA_DIR = 0xFF;
	
//	if(incXAdd == INCREMENT_X)					// display x-address is incremented automatically
//		ks0108Coord.x++;
//	else	
	    ks0108GotoXY(ks0108Coord.x, ks0108Coord.y);
	
	if(ks0108Inverted)
		data = ~data;
	return data;
}

void ks0108Init(unsigned char invert)
{  
ks0108Coord.x = 0;
ks0108Coord.y = 0;
ks0108Coord.page = 0;	
ks0108Inverted = invert;
LCD_CMD_DIR = 0x1F;								// command port is output
ks0108WriteCommand(LCD_ON, CHIP1);				// power on
ks0108WriteCommand(LCD_ON, CHIP2);
ks0108WriteCommand(LCD_DISP_START, CHIP1);		// display start line = 0
ks0108WriteCommand(LCD_DISP_START, CHIP2);
ks0108ClearScreen();							// display clear
ks0108GotoXY(0,0);
}


// Graphic Functions

void ks0108DrawLine(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char color)
{
unsigned char length, i, y, yAlt, xTmp, yTmp;
long int m;
// vertical line
if(x1 == x2)
	{
	// x1|y1 must be the upper point
	if(y1 > y2)
		{
		yTmp = y1;
		y1 = y2;
		y2 = yTmp;
		}
		ks0108DrawVertLine(x1, y1, y2-y1, color);
	// horizontal line
	}
else if(y1 == y2)
	{	
	// x1|y1 must be the left point
		if(x1 > x2)
			{
			xTmp = x1;
			x1 = x2;
			x2 = xTmp;
			}
		ks0108DrawHoriLine(x1, y1, x2-x1, color);
	// schiefe line :)
	}
else
	{
	// angle >= 45
	if((y2-y1) >= (x2-x1) || (y1-y2) >= (x2-x1))
		{
		// x1 must be smaller than x2
		if(x1 > x2)
			{
			xTmp = x1;
			yTmp = y1;
			x1 = x2;
			y1 = y2;
			x2 = xTmp;
			y2 = yTmp;
			}
		length = x2-x1;		// not really the length :)
		m = (((int)y2-y1)*200)/length; 
		yAlt = y1;	
		for(i=0; i<=length; i++)
			{
			y = ((m*i)/200)+y1;	
			if((m*i)%200 >= 100)
					y++;
			else if((m*i)%200 <= -100)
					y--;
			ks0108DrawLine(x1+i, yAlt, x1+i, y, color);
				if(length <= (y2-y1) && y1 < y2)
					yAlt = y+1;
				else if(length <= (y1-y2) && y1 > y2)
					yAlt = y-1;
				else
					yAlt = y;
			}
		// angle < 45
		}
	else
		{
		 // y1 must be smaller than y2
		 if(y1 > y2)
		 	{
			xTmp = x1;
			yTmp = y1;
			x1 = x2;
			y1 = y2;
			x2 = xTmp;
			y2 = yTmp;
			}
			length = y2-y1;  
			m = (((int)x2-x1)*200)/length;
			yAlt = x1;
			for(i=0; i<=length; i++)
				{
				y = ((m*i)/200)+x1;
				if((m*i)%200 >= 100)
					y++;
				else if((m*i)%200 <= -100)
					y--;
				ks0108DrawLine(yAlt, y1+i, y, y1+i, color);
				if(length <= (x2-x1) && x1 < x2)
					yAlt = y+1;
				else if(length <= (x1-x2) && x1 > x2)
					yAlt = y-1;
				else
					yAlt = y;
				}
		}
	}
}                                

void ks0108FillRect(unsigned char x, unsigned char y, unsigned char width, unsigned char height, unsigned char color)
{
unsigned char mask, pageOffset, h, i, data;
height++;
pageOffset = y%8;
y -= pageOffset;
mask = 0xFF;
if(height < 8-pageOffset)
	{
	mask >>= (8-height);
	h = height;
	}
else 
	{
	h = 8-pageOffset;
	}
mask <<= pageOffset;

ks0108GotoXY(x, y);
for(i=0; i<=width; i++)
	{
	data = ks0108ReadData();
	if(color == BLACK)
		{
		data |= mask;
		}
	else
		{
		data &= ~mask;
		}
	ks0108WriteData(data);
	}

while(h+8 <= height)
	{
	h += 8;
	y += 8;
	ks0108GotoXY(x, y);

	for(i=0; i<=width; i++)
		{
		ks0108WriteData(color);
		}
	}

if(h < height)
	{
	mask = ~(0xFF << (height-h));
	ks0108GotoXY(x, y+8);

	for(i=0; i<=width; i++)
		{
		data = ks0108ReadData();

		if(color == BLACK)
			{
			data |= mask;
			}
		else
			{
			data &= ~mask;
			}
		ks0108WriteData(data);
		}
	}
}           

// Font Functions
unsigned char ks0108ReadFontData(flash unsigned char* ptr)
{
return (*ptr);
}

void ks0108SelectFont(flash unsigned char* font, ks0108FontCallback callback, unsigned char color)
{
ks0108Font = font;
ks0108FontRead = callback;
ks0108FontColor = color;
}

int ks0108PutChar(char c)
{
unsigned char i,j,b;
unsigned char width = 0;
unsigned char height; 
unsigned char bytes;
unsigned char page;
unsigned char data;
unsigned char firstChar;
unsigned char charCount;

unsigned long int index = 0;
unsigned char x,y;

height = (*ks0108FontRead)(ks0108Font+FONT_HEIGHT);
bytes = (height+7)/8;
firstChar = (*ks0108FontRead)(ks0108Font+FONT_FIRST_CHAR);
charCount = (*ks0108FontRead)(ks0108Font+FONT_CHAR_COUNT);
x = ks0108Coord.x;
y = ks0108Coord.y;
if(c < firstChar || c >= (firstChar+charCount))
	{
	return 1;
	}
c-= firstChar;

// read width data, to get the index 
for(i=0; i<c; i++)
	{
	index += (*ks0108FontRead)(ks0108Font+FONT_WIDTH_TABLE+i);
	}
index = index*bytes+charCount+FONT_WIDTH_TABLE;
width = (*ks0108FontRead)(ks0108Font+FONT_WIDTH_TABLE+c);

// last but not least, draw the character 
for(b=0; b<bytes; b++)
	{
	page = b*width;
	for(j=0; j<width; j++)
		{
		data = (*ks0108FontRead)(ks0108Font+index+page+j);

		if(height < (b+1)*8)
			{
			data >>= (b+1)*8-height;
			}

		if(ks0108FontColor == BLACK)
			{
			ks0108WriteData(data);
			}
		else
			{
			ks0108WriteData(~data);
			}
		}
	// 1px gap between chars
	if(ks0108FontColor == BLACK)
		{
		ks0108WriteData(0x00);
		}
	else
		{
		ks0108WriteData(0xFF);
		}
	ks0108GotoXY(x, ks0108Coord.y+8);
	}
ks0108GotoXY(x+width+1, y);

return 0;
}

void ks0108Puts(char* str)
{
int x;
x = ks0108Coord.x;
while(*str != 0)
	{
	if(*str == '\n')
		{
		ks0108GotoXY(x, ks0108Coord.y+ks0108FontRead(ks0108Font+FONT_HEIGHT));
		}
	else
		{
		ks0108PutChar(*str);
		}
		str++;
	}
}





void main(void)
{
// Declare your local variables here
char buffer[65];
// Input/Output Ports initialization
// Port A initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTA=0x00;
DDRA=0x00;

// Port B initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTB=0x00;
DDRB=0x00;

// Port C initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTC=0x00;
DDRC=0x00;

// Port D initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTD=0x00;
DDRD=0x00;

// Port E initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTE=0x00;
DDRE=0x00;

// Port F initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTF=0x00;
DDRF=0x00;

// Port G initialization
// Func4=In Func3=In Func2=In Func1=In Func0=In 
// State4=T State3=T State2=T State1=T State0=T 
PORTG=0x00;
DDRG=0x00;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: Timer 0 Stopped
// Mode: Normal top=FFh
// OC0 output: Disconnected
ASSR=0x00;
TCCR0=0x00;
TCNT0=0x00;
OCR0=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer 1 Stopped
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// OC1C output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
// Compare C Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;
OCR1CH=0x00;
OCR1CL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer 2 Stopped
// Mode: Normal top=FFh
// OC2 output: Disconnected
TCCR2=0x00;
TCNT2=0x00;
OCR2=0x00;

// Timer/Counter 3 initialization
// Clock source: System Clock
// Clock value: Timer 3 Stopped
// Mode: Normal top=FFFFh
// Noise Canceler: Off
// Input Capture on Falling Edge
// OC3A output: Discon.
// OC3B output: Discon.
// OC3C output: Discon.
// Timer 3 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
// Compare C Match Interrupt: Off
TCCR3A=0x00;
TCCR3B=0x00;
TCNT3H=0x00;
TCNT3L=0x00;
ICR3H=0x00;
ICR3L=0x00;
OCR3AH=0x00;
OCR3AL=0x00;
OCR3BH=0x00;
OCR3BL=0x00;
OCR3CH=0x00;
OCR3CL=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
// INT2: Off
// INT3: Off
// INT4: Off
// INT5: Off
// INT6: Off
// INT7: Off
EICRA=0x00;
EICRB=0x00;
EIMSK=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x00;
ETIMSK=0x00;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
SFIOR=0x00;
ks0108Init(0); 
ks0108SelectFont(Dial_10, ks0108ReadFontData,BLACK);  
sprintf(buffer,"page0,page0");   
ks0108GotoXY(5,0);
ks0108Puts(buffer); 
sprintf(buffer,"page7");   
ks0108GotoXY(5,56);
ks0108Puts(buffer); 
while (1)
      {
      // Place your code here

      };
}
